use rocket::{
	form::{Contextual, Form},
	http::CookieJar,
	response::Redirect,
	tokio::task,
};
use rocket_dyn_templates::{context, Template};

use crate::{
	auth::pw,
	db::{schema::DollTagsDb, user},
	routes::error_handlers::{PageResponse, PageResult},
};

#[derive(Debug, FromForm)]
pub struct AuthForm<'a> {
	#[field(validate=len(2..=256))]
	pub username: &'a str,
	#[field(validate=len(8..))]
	pub password: &'a str,
}

#[get("/login")]
pub fn show_login() -> Template {
	todo!("meow")
}

#[post("/login", data = "<form>")]
pub async fn handle_login(
	db: DollTagsDb,
	form: Form<Contextual<'_, AuthForm<'_>>>,
	cookies: &CookieJar<'_>,
) -> PageResult {
	let miss = || PageResponse::Page(Template::render("login", context! {failure: true}));

	let values = match &form.value {
		None => return Ok(miss()),
		Some(v) => v,
	};

	let user_in_db = user::get(db, &values.username).await?;
	let user = match user_in_db {
		None => return Ok(miss()),
		Some(v) => v,
	};

	let password = String::from(values.password);
	let right_password =
		task::spawn_blocking(move || pw::verify(&password, &user.password)).await??;

	if right_password && user.enabled {
		cookies.add_private(("user_id", user.id.to_string()));
		Ok(Redirect::to("/").into())
	} else {
		Ok(miss())
	}
}

#[get("/register")]
pub fn show_register() -> Template {
	todo!("meow")
}

#[post("/register", data = "<_form>")]
pub async fn handle_register(
	_db: DollTagsDb,
	_form: Form<AuthForm<'_>>,
	_cookies: &CookieJar<'_>,
) -> PageResult {
	todo!("meow")
}
